/*##### fieldbuttonsWithComments.js; part of ###Field Buttons, by Robin Richmond
This is a code-commented copy of a Javascript Library that implements "Field Buttons"
in TNG web pages. For additional information about Field Buttons, see
- The Field Buttons article in the TNG Wiki, and
- The MS Word document fieldbuttons.doc in the Field Buttons mod subfolder.

The Javascript code in this file is identical to the Javascript code
in the "real" Field Buttons Javascript library, fieldbuttonsmod.js.

***********************
Revision History

v2a 4 May 2019
- Added two external functions that hide Field Buttons global variables
- Combined two legacy functions into one
- Now looks for the field button active flags in $rrconfig['FieldButtonStatus']
  rather than $tngconfig[...]
- Initializes the status of all field buttons to  "active" so that they work if
  the mod has been installed but the status system variable has not been defined.
v2 9 Jan 2018
- Now allows multi-character labels
- Maps from field button names to numbers, and from numbers to names */

//*****************************************************************************
//********************  GLOBAL JAVASCRIPT VARIABLES  **************************
//*****************************************************************************
//GLOBAL JAVASCRIPT VARIABLES
/* rrButtonsLabels is made of of one-character labels for each type of field button.
It is initialized when the parent program calls the library function rrButtonsSystemSettings
to pass theTNG system variable $tngconfig('rrButtonsLabels') to the library.
$tngconfig('rrButtonsLabels') is a comma-delimited strings, but the library function
rrButtonsCreateThem changes rrButtonsLabels to an array (a vector). */
var rrButtonsLabels = "";
var rrButtonsMap = Array(); //Maps 'X','R','D', and 'A' to the actual button values
var rrButtonsMap2 = Array(); //Maps the actual button values to 'X','R','D', and 'A'

/*rrButtonsFlags is a string that hold active-inactive flags (1/0) for each button type.
It is given a default value here, but is really initialized when the parent program calls
the library function rrButtonsSystemSettings to pass the language string 
$text['rrfb-buttonlabels'] to the library.
It is later merged into rrbuttonsLabels so that inactive buttons have a label values of 0. */
var rrButtonsFlags = '1,1,1,1';

/*rrButtonsValues[formfieldID][buttontype] will hold the values assigned to a form field
by a button. The values for X and R buttons are populated in rrButtonsCreateThem as the
relevant button classes are encountered in form fields. The values for D and A buttons
are populated by the parent program in a function that they store in the array
rrButtonsFinalSetupFunctions */
var rrButtonsValues = Array();

/*rrButtonsDocumentReadyFunctions[ii] is a numbered array for functions defined by the
parent program, and that we will call in our Document.ready function, before creating
the buttons. These functions are used by the parent program to create ID's
and/or field button classes for form fields when a TNG mod uses Javascript for this purpose
rather than modifying the source code that defines the form field HTML in the first place.
It is an array only because more than one TNG mod could define form buttons in the same
TNG program, and each mod might need to define its own function. */
XYZ
  The most common use of a 'before' function is to add field buttons to HTML forms (at run-time)
  by adding the field button classnames (and, often, an ID) to the form field's HTML element.
  The only way to guarantee that those actions are performed before the library needs
  those HTML attributes is for the programmer to write a function, and then pass that
  function to the library
  rrButtonsPushFunction to pass that function to the library.
  circumstance would be a parent program to put them in a function that
  is sent to the library 
	array (declared globally by the library) so that the library can execute them. */
var rrButtonsDocumentReadyFunctions = Array();

/* rrButtonsFinalSetupFunctions[formfieldID] is a numbered array (for each form field) that
is filled by the parent program to initialize rrButtonsValues & rrButtonsOnClickFunctions,
and, typically, to call rrButtonsSetLabels().
We will call it from our Document.ready function, after we call the functions in
rrButtonsDocumentReadyFunctions, and after we then declare the arrays in the [formfieldID]
subscripts in rrButtonsValues and rrButtonsOnClickFunctions. */
var rrButtonsFinalSetupFunctions = Array();

/* rrButtonsOnClickFunctions[formfieldID]
If a form field with Field Buttons also has an OnChange event, that OnChange event must also be invoked
by the Field Buttons when they change the field value.  To accomplish this, the parent program must put
the field's OnChange event handler (or a function that calls the event handler) in a global array that
the Field Buttons library calls from its Field Button OnClick handlers. */
var rrButtonsOnClickFunctions = Array();

//*****************************************************************************
//************************  EXTERNAL FUNCTIONS ********************************
//*****************************************************************************
// All of these external functions just supply data to the library's variables.

//////////////////////////////////////////////////
// External function that supplies the field buttons library with
// the two TNG site-dependent values that it needs.
function rrButtonsSystemSettings (theLabels, theFlags) {
	rrButtonsLabels = theLabels;
	rrButtonsFlags = theFlags;
}
//////////////////////////////////////////////////
//External function that pushes functions defined by the parent
//program onto one of three internal field buttons arrays.
//In previous versions of the library, the parent program
//had to refer to internal library variables directly.
function rrButtonsPushFunction (beforeOrAfter, theFunction) {
	if (beforeOrAfter == 'before')
		rrButtonsDocumentReadyFunctions.push(theFunction);
	else if (beforeOrAfter == 'after')
		rrButtonsFinalSetupFunctions.push(theFunction);
	else rrButtonsOnClickFunctions.push(theFunction);
}
//////////////////////////////////////////////////
//External function that initializes the values that are saved to a field
// by a D or A button.  In previous versions of the library, the parent
// program had to reference the variable rrButtonsValues directly.
function rrButtonsSetValue (fieldID, buttonType, theValue) {
	rrButtonsValues[fieldID][buttonType] = theValue;
}
//*****************************************************************************
//********************  DEPRECATED EXTERNAL FUNCTIONS *************************
//*****************************************************************************
//////////////////////////////////////////////////
//Deprecated external function that supplies the button LABELS, which
//can vary among TNG sites. In Field Buttons v3, this function has been
//superseded by rrButtonsSystemSettings. This function is retained in the
//library in case a mod that defines field buttons and predated Field
//Buttons mod v3 is still installed.
function rrButtonsSetLabels(theLabels) {
	rrButtonsLabels = theLabels;
}
//////////////////////////////////////////////////
//Deprecated external function that supplies the button ACTIVE FLAGS,
//which can vary among TNG sites. In Field Buttons v3, this function has
//been superseded by rrButtonsSystemSettings. This function is retained
//in the library in case a mod that that defines field buttons and
//predated Field Buttons mod v3 is still installed.
function rrButtonsSetFlags(theFlags) {
	rrButtonsFlags = theFlags;
}

//*****************************************************************************
//************************  INTERNAL FUNCTIONS ********************************
//*****************************************************************************
////////////////////////////////////////////////////
///////////// document.ready function///////////////
$(function() {
	//See the Word document for an overview...

  /* Call the so-called 'Before' document.ready functions that were defined in the parent
  program and passed to the field buttons library through a call to
  rrButtonsPushFunction('before',...).*/
	for (ii=0; ii<rrButtonsDocumentReadyFunctions.length; ii++)
		rrButtonsDocumentReadyFunctions[ii]();
	//For each form field that has a Field Button, get its ID, and initialize two arrays for each field.
	$('.rrbtnx,.rrbtnr,.rrbtnd,.rrbtna').each(function() {
		var id = $(this).attr('id');
		rrButtonsValues[id] = Array();
		rrButtonsOnClickFunctions[id] = Array();
	});

	//Call the Final Setup functions defined by the parent program
	for (ii=0; ii<rrButtonsFinalSetupFunctions.length; ii++)
		rrButtonsFinalSetupFunctions[ii]();

	//Now it is safe for the library to set up the Field Buttons in the global arrays
	//that have been declared by the library.
	rrButtonsCreateThem();
});

////////////////////////////////////////////////////////////////////
//Internal function called by the internal document.ready function to populate global Field Button arrays
function rrButtonsCreateThem() {
	//Make sure that at least one button is not flagged as inactive
	if (rrButtonsFlags == '0,0,0,0') return;
	//If the button flags have not been initialized in the configuration options,
	//treat all of them as if they were enabled.
	if (!rrButtonsFlags) rrButtonsFlags = "1,1,1,1";
	//Default button labels.
	if (!rrButtonsLabels) rrButtonsLabels = "X,R,D,A";
	//Convert entities to characters
	//We have to do this since they get converted to characters when they are
	//written to the HTML page, and we read them back for comparison.
	var textArea = document.createElement('textarea');
	textArea.innerHTML = rrButtonsLabels;
	rrButtonsLabels = textArea.value.split(',');
	rrButtonsFlags = rrButtonsFlags.split(',');

	//Merge the Flags into the Labels so that inactive buttons have the label 0
	for (bb=0; bb<4; bb++)
		if (!rrButtonsFlags[bb]) rrButtonsLabels[bb] = 0;
	//Clear the values array for the D and A buttons if those buttons are disabled.
	for (id in rrButtonsValues) {
		if (!rrButtonsLabels[2]) rrButtonsValues[id]['D'] = Array();
		if (!rrButtonsLabels[3]) rrButtonsValues[id]['A'] = Array();
	}
	//Set up the two maps between actual button values and 'X','R','D', and 'A'
	rrButtonsMap['X'] = rrButtonsLabels[0];
	rrButtonsMap['R'] = rrButtonsLabels[1];
	rrButtonsMap['D'] = rrButtonsLabels[2];
	rrButtonsMap['A'] = rrButtonsLabels[3];
	rrButtonsMap2[rrButtonsMap['X']] = 'X';
	rrButtonsMap2[rrButtonsMap['R']] = 'R';
	rrButtonsMap2[rrButtonsMap['D']] = 'D';
	rrButtonsMap2[rrButtonsMap['A']] = 'A';

	/* For each element (i.e. each form field) with a Field Button class,
	a. Create an onchange event handler for the form field.
	b. Wrap a span element around the form field to keep the Field Buttons attached to it.
	c. Create the appropriate physical field buttons,
	d. Initialize the display of the field buttons. */
	$('.rrbtnx,.rrbtnr,.rrbtnd,.rrbtna').each(function() {
		var id = $(this).attr('id');
		//a. Create an onchange event handler for the form field to make sure that the
		//   appropriate Field Buttons are displayed, given the new form field value.
		$(this).on('change', function(event) {
			rrButtonsDisplayToggle(event.target.id);
			return false;
		}); //End of form field onchange event handler

		//b. Wrap the form field in a span element that forbids wrapping to
		//   keep the form field and the field buttons together.
		$(this).wrap("<span class='nw normal'></span>")

		//c. Create the Field Buttons for this form field.
		/*They are placed after the form field, inside the wrapper. We create one
		Field Button for each Field Button class defined in that form field.
		The Field Buttons ID's use the form field ID, with numbered suffixes,
		starting with 1, and to a maxium of 4 (since there are 4 Field Button types).
		Field Buttons are not permanently tied to a Field Button type. Instead,
		the rrButtonsDisplayToggle function (which takes a form field ID as its input
		parameter):
		 - Steps through the field button types,
		 - Decides which Field Button labels should be displayed,
		 - Assigns visible Field Buttons labels to the actual Field Buttons
		   (starting with button #1, immediately to the right of the form field),
		 - And then assigns blank lables to the rest of the actual Field Buttons,
		 so that a form field's field buttons always occupy the same horiton spaces
		 in the form, and the visible buttons are always adjacent to the form field.
		 For example, a form might load with
		     Cemetery Name: [Shiloh Cemetery______]X_
		   where 'X' represents a Clear Field Button and the underscore after the X
		   represents a hidden Restore/Refresh Field Button. (Restore/Refresh field
		   buttons are always hidden when the page is loaded, since they come into
		   play only when the value changes from the initial value.

		 The field buttons displayed for a field are updated
		 - after the Field Buttons for one field are created,
		 - every time any Field Button associated with a form field is clicked, and
		 - every time a form field with a Field Button changes.

		We'll create an HTML string that contains the HTML code for each button to
		be defined for this form field, and do one jQuery.after call to create all
		of the buttons for this form field at once.
		The buttons for a form all begin with the ID of the field, then 'fb', then the
		button letter (X,R,D,A).
		The variable buttonTemplate is a template for a button's <input> element.
		The string XXX will be replaced with the button letter. */
		var html = '';
		var buttonTemplate = "<a href='#' class='rrbtn' id='" +id+ "rrfbXXX'></a>";

		//X - Clear the field
		if ($(this).hasClass('rrbtnx') && rrButtonsLabels[0]) {
			html += buttonTemplate.replace('XXX','X');
			//X buttons will always store '' in the field.
			rrButtonsValues[id]['X'] = '';
		}
		//R - Reset/Reload buttons that reload the initial value of a field.
		/*If the initial value happens to be the same as the X, D, or A button values,
		then the R button is ostensibly redundant, but we don't care.
		That is, if the field is defined to say "add an R" button, then the user is
		supposed to be able to restore the initial value, whatever it was, so we
		WILL supply an R button. */
		if ($(this).hasClass('rrbtnr') && rrButtonsLabels[1]) {
			html += buttonTemplate.replace('XXX','R');
			//R button will always store the intial field value in the field.
			rrButtonsValues[id]['R'] = $(this).val();
		}
		//D - Default buttons that load an externally-defined default value.
		//If the D and A buttons happen to have the same value, they are redundant,
		//and we'll skip the D button (if the field has an A button).
		if ($(this).hasClass('rrbtnd') && rrButtonsLabels[2] &&
			!($(this).hasClass('rrbtna') && rrButtonsLabels[3]
				&& rrButtonsValues[id]['D'] == rrButtonsValues[id]['A'] )) {
			html += buttonTemplate.replace('XXX','D');
		}
		//A - 'All Items' buttons that load a predefined value
		if ($(this).hasClass('rrbtna') && rrButtonsLabels[3]) {
			html += buttonTemplate.replace('XXX','A');
		}
		//Now, create the buttons for this field
		$(this).after(html);

		//d. Initialize the display of this field's field buttons
		rrButtonsDisplayToggle(id);
	}); //End of .each loop through the form fields that have field buttons.

	//Step 4 - Create the onchange events handlers for the field buttons themselves.
	//This function uses the button's label (its html value), to decide which
	//value to assign to the data field.
	$('.rrbtn').on('click', function(event) {
		var id = event.target.id.slice(0,-5); //The data field's id
		var label = $(this).html();
		if (label == '&nbsp;') return false;
		label = rrButtonsMap2[label];
		if (label in rrButtonsValues[id])
			$('#'+id).val(rrButtonsValues[id][label]);
		if (id in rrButtonsOnClickFunctions) {
			for (ii=0; ii<rrButtonsOnClickFunctions[id].length; ii++)
				rrButtonsOnClickFunctions[id][ii]();
		}
		//Set the display status of each of this form field's Field Buttons.
		rrButtonsDisplayToggle(id);
		return false;
	}); // end of Field Button onclick event handler
} //End function rrButtonsCreateThem

/////////////////////////////////////////////////////////////////////////
// External function to set the display status of a form field's Field Buttons,
// based on the field's value.
// This function should be called by parent program Javascript functions that
// the value of a form field that has Field Buttons.
//Input parameter - the ID of the form field.
function rrButtonsDisplayToggle(id) {
	var buttonValue;
	var hashid = '#'+id+'rrfb'; //Prefix for field button id's
	var fieldval = $('#'+id).val()

	//For each button type for this field, if the current field value is not
	//the buttons's field value, then display the button's letter.
	//Otherwise, leave the button blank.
	for (btype in rrButtonsValues[id]) {
		buttonValue = rrButtonsMap[btype];
		if (rrButtonsValues[id][btype] != fieldval) {
			$(hashid+btype).html(rrButtonsMap[btype]);
			$(hashid+btype).css('padding','0');
			$(hashid+btype).css('border-width','1');
		} else {
			//Build a value with the same # of &nbsp;'s as the normal button value
			buttonValue = '~~~~~~~~~~~~~~~'.substr(0,rrButtonsMap[btype].length).replace(/~/g,'&nbsp;')
			$(hashid+btype).html(buttonValue);
			$(hashid+btype).css('padding','1');
			$(hashid+btype).css('border-width','0');
		}
	}
} //End function rrButtonsDisplayToggle
// End of file
